#include <iostream>
#include <string>

using namespace std;

// allocates an array of N int and fills it with the Fibonacci sequence
//   Sequence is defined by F[i] = F[i-1] + F[i-2] with F[0]=F[1]=1
int* makeFibonacciArray(unsigned N)
{
    int *fibArray = new int[N];
    fibArray[0]=1;
    fibArray[1]=1;
    for(unsigned j=2;j<N;++j)
        fibArray[j] = fibArray[j-1]+fibArray[j-2];
    return fibArray;
}

// allocates an array of N ints and fills it with the sequence N!
//   Sequence is defined by F[i] = i*F[i-1] with F[0]=1
int* makeFactorialArray(unsigned N)
{
    int *facArray = new int[5];

    facArray[0]=1;
    for(unsigned i=1; i< N; ++i)
        facArray [i] = i*facArray[i-1];
    return facArray;
}

// Prints N elements of a sequence, giving a title line with the sequence name and N
// One line per element, with an indent of 2 spaces
void printSequence(string name,int* sequence,int N)
{
    cout << "First " << N << " element of sequence \"" << name << "\" are: " << endl;
    for(unsigned i=0;i<N;++i)
        cout << "  " << sequence[i] << endl;
}

// creates an array holding one row (specified by r) of pascal's triangle (r+1 elements)
//   P[i] = (r choose i) for i = [0,r]
int* makePascalArray(unsigned r)
{
    int *p = new int[r+1];

    // fill the first half [0, ceil(r/2)] using the definition
    for(unsigned i=0;i<=(r+1)/2; ++i)
    {
        p[i]=1;
        for(unsigned j=r;j > r-i;--j)
            p[i] *= j;
        for(unsigned j=1;j <= i;++j)
            p[i] /= j;
    }

    // fill second half by symmetry with first half p[i] = p[r-i]
    for(unsigned i=r;i > (r+1)/2; --i)
        p[i] = p[r-i];
    return p;
}

int main(int argc,char **argv)
{
    int *seq = makeFibonacciArray(10);

    printSequence("fibonacci",seq,10);

    seq = makeFactorialArray(10);

    printSequence("factorial",seq,10);

    int *fac = seq;

    printSequence("pascal5",makePascalArray(5),6);

    int *yet_another = makeFactorialArray(5);

    delete[] fac;
    delete[] seq;
}
